/*
  DataWriteThread 
  
  The workhorse of the program, mantains probeData entries and saves them into a file in a separate thread;
  
*/

class DataWriteThread extends Thread {
  ArrayList<probeData> dataEntries;
  PrintWriter file;
  Serial ser;
  String startTimeStamp;
  probeData lastData;
  
  // -- Constructor 
  DataWriteThread(String filename, Serial ser) {
    file = createWriter(filename);
    this.ser  = ser;
    dataEntries = new ArrayList<probeData>();
    startTimeStamp = getTimeString();

    // dataEntries.ensureCapacity(200);
  }
  
  // -- Runs file writing routine 
  void run() {
    file.println(String.format("---- MEASUREMENT [ %s : %s ]----", startTimeStamp, getTimeString()));
    while (dataEntries.size() > 0) {
      writeEntry(dataEntries.get(0));
      dataEntries.remove(0);
    }
    file.flush();
  }
  
  // -- writes one of entries
  void writeEntry(probeData entry) {
    file.println(entry.getCSVDataLine());
  }
  
  // -- getters/ setters
  // -- get last valid and full entry 
  float[] getLastData() {
    if (lastData != null)
      return lastData.getData();
    else
      return new float[]{0.0, 0.0, 0.0};
  }
  boolean hasData() {
    return !dataEntries.isEmpty() ;
  }

  void enterEntry() {
    if (!hasData())
      startTimeStamp = getTimeString();
    dataEntries.add(new probeData(ser));
    println("[MAIN]"+"Creating entry");
  }
  
  // -- Serial processing
  String buff = "";
  char prefix  = '\0';
  boolean hasDecimalPoint = false;
  boolean hasDecimals = false;

  void readChr(char ch) {
    //-- Prefix 
    if (Character.isLetter(ch)) {
      //Removes prematurely terminated message
      if (buff.length() > 1) {
        buff="";
        println("[DATA]: Removing garbage message");
      }
      prefix = ch;
    }
    // -- Digits
     else if (Character.isDigit(ch)) {
      if (hasDecimalPoint) {
        hasDecimals = true;
      }
      buff += ch;
    }
    //-- Decimal Point
    else if (ch == '.') {
      if (prefix != '\0') {
        if (buff.length() > 0)
          buff += ch;
        else
          buff += '0'+ch;
        hasDecimalPoint = true;
      }
    }
    //-- Endline
    else if (ch == '\n') {
      if (dataEntries.isEmpty())
        enterEntry();
        
      probeData entry = dataEntries.get(dataEntries.size() -1);
      entry.readParseSerialLine(prefix, buff);
      buff = "";
      prefix = '\0';
      hasDecimalPoint = false;
      hasDecimals = false;
      
      if (entry.isFull()) {
        lastData = entry;
        enterEntry();
      }
    }
  }
}
