import processing.serial.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/*
 * ProjMeasure - SE Project Measurements saver
 *
 * Displays the recieved data from an arduino reciever into a small GUI and writes them into a .csv file
 * Also controls the probe servo
 */


// Constants
static final long MEAS_TOT = 10000;

// Objects 
Serial serial;
DataWriteThread writer;
DateTimeFormatter dtf;

// State
boolean measuring;
long measureTimeout = Long.MAX_VALUE;

// -- Setup
void setup() {
  size(600, 500);
  dtf = DateTimeFormatter.ofPattern("HH-MM-SS");
  serial = new Serial(this, Serial.list()[0], 9600);
  writer = new DataWriteThread(String.format("PHMeasureData-%s.csv", getTimeString()), serial);
  Runtime.getRuntime().addShutdownHook(writer);
}

// -- Drawing the gui 
void drawGUI() {
  background(0);
  strokeWeight(5);
  stroke(255);
  noFill();
  rect(10, 10, width-20, height-20);
  textSize(50);
  fill(255);
  float data[] = writer.getLastData();
  pushMatrix();
  translate(40,80);
  text(measuring ? "[X]" : "[  ]", 0,0);
  
  translate(0,50);
  text("X:",0,0);
  text(data[0],50,0);
  
  translate(0,50);
  text("Y:",0,0);
  text(data[1],50,0);
  
  translate(0,50);
  text("P:",0,0);
  text(data[2],50,0);
  popMatrix();
}

void draw() {
  if (measuring) {                               // Measurment timeout
    if (measureTimeout < millis())
      measuring = false;
  } else {
    if (writer.hasData() && !writer.isAlive()) { // Write data when not measuring 
      writer.run();
      println("Writing to file");
    }
  }if (serial.available() > 0) {                 // Getting data from serial
    if (!writer.isAlive())
      writer.readChr(serial.readChar());
    measuring = true;
    measureTimeout = millis() + MEAS_TOT;
  }
  
  drawGUI();
}

// -- Controlls for the probe servo
void keyPressed() {
  print("Start Measurement");
  serial.write("aaaa");
}

// -- returns an accurate timestamp 
String getTimeString() {
  return dtf.format(LocalDateTime.now());
}
